import torch
import os
import random
import pickle
import numpy as np

def update_topk_acc(acc,y_found,target_labels,nb_iter,k):
    _,l_found = torch.topk(y_found,k)
    target_labels = target_labels.unsqueeze(1).repeat(1,k)
    acc = acc + torch.eq(l_found,target_labels).float().mean()*k/nb_iter
    return acc

def get_balanced_benchmark(nb_corruption,nb_category, corruption_clusters):
    bench = []
    corruption_clusters = dict(corruption_clusters)
    if nb_corruption%nb_category !=0:
        print("print error in nb corruptions")
        exit()
    nb_corruption_per_cat = int(nb_corruption/nb_category)
    for _ in range(nb_category):
        cluster_name = random.choice(list(corruption_clusters.keys()))
        cluster = corruption_clusters[cluster_name]
        for _ in range(nb_corruption_per_cat):
            cur_cc = random.choice(cluster)
            while cur_cc in bench:
                cur_cc = random.choice(cluster)
            bench.append(cur_cc)
        corruption_clusters.pop(cluster_name)
    return bench


def get_representation_bench(benchmark, reverse_clusters):
    nb_clusters = len(list(reverse_clusters.values()))
    representation = dict(zip(list(reverse_clusters.values()), [0]*nb_clusters))
    for cc in benchmark:
        cur_cluster = reverse_clusters[cc][0]
        representation[cur_cluster] = representation[cur_cluster]+1
    return representation

def substitute_cc(benchmark, reverse_clusters, corruption_clusters):
    new_benchmark = list(benchmark)
    ori_lenght_bench = len(benchmark)
    representation = get_representation_bench(new_benchmark, reverse_clusters)
    list_clusters = list(representation.keys())
    random.shuffle(list_clusters)
    for k in list_clusters:
        if representation[k] == 0:
            representation.pop(k)
    mean_nb_cc = np.mean(list(representation.values()))
    random.shuffle(new_benchmark)
    for random_cc in list(new_benchmark):
        random_cat = reverse_clusters[random_cc][0]
        if representation[random_cat] != 1 and representation[random_cat]<=mean_nb_cc:
            new_benchmark.remove(random_cc)
            break
    representation = get_representation_bench(new_benchmark, reverse_clusters)
    random.shuffle(list_clusters)
    for random_cat in list_clusters:
        for random_cc in list(corruption_clusters[random_cat]):
            if random_cc not in new_benchmark and representation[random_cat]>=mean_nb_cc :
                break
        if random_cc not in new_benchmark and representation[random_cat]>=mean_nb_cc :
            new_benchmark.append(random_cc)
            break
    if len(new_benchmark) != ori_lenght_bench:
        #"no possible substitution"
        return benchmark
    else :
        return new_benchmark
